# TMVA linear discriminators

import ROOT
 
ROOT.TMVA.Tools.Instance()
 
# note that it seems to be mandatory to have an
# output file, just passing None to TMVA::Factory(..)
# does not work. Make sure you don't overwrite an
# existing file.

fin = ROOT.TFile.Open("ntuple.root")
ntuple = fin.Get('ntuple')
ntuple.Print()

fout = ROOT.TFile("TMVA.root","RECREATE")

dataloader = ROOT.TMVA.DataLoader("dataset");
factory = ROOT.TMVA.Factory("TMVAClassification", fout,
                            ":".join([
                                "!V",
                                "!Silent",
                                "Color",
                                "DrawProgressBar",
                                "Transformations=I;D;P;G,D",
                                "AnalysisType=Classification"]
                                     ))

dataloader.AddVariable("x","F")
dataloader.AddVariable("y","F") 
 
dataloader.AddSignalTree(ntuple,1.0)
dataloader.AddBackgroundTree(ntuple,1.0)
 
# cuts defining the signal and background sample
sigCut = ROOT.TCut("signal > 0.5")
bgCut = ROOT.TCut("signal <= 0.5")
 
dataloader.PrepareTrainingAndTestTree(sigCut,   # signal events
                                   bgCut,    # background events
                                   ":".join([
                                        "nTrain_Signal=5000",
                                        "nTrain_Background=5000",
                                        "SplitMode=Random",
                                        "NormMode=NumEvents",
                                        "!V"
                                       ]))

#method = factory.BookMethod(dataloader,ROOT.TMVA.Types.kCuts, "Cuts",
                   #":".join([
                       #"!H",
                       #"!V",
                       #"FitMethod=MC:EffSel:SampleSize=200000:VarProp=FSmart",
                       #]))
 

#method = factory.BookMethod(dataloader,ROOT.TMVA.Types.kCuts, "CutsPCA",
                   #":".join([
                       #"!H",
                       #"!V",
                       #"FitMethod=MC:EffSel:SampleSize=200000:VarProp=FSmart:VarTransform=P",
                       #]))
 

#method = factory.BookMethod(dataloader,ROOT.TMVA.Types.kCuts, "CutsD",
                   #":".join([
                       #"!H",
                       #"!V",
                       #"FitMethod=MC:EffSel:SampleSize=200000:VarProp=FSmart:VarTransform=D",
                       #]))

factory.BookMethod(dataloader, ROOT.TMVA.Types.kFisher, 'Fisher',
        '!H:!V:Fisher') 

#factory.BookMethod(dataloader, ROOT.TMVA.Types.kFisher, 'FisherD',
#        '!H:!V:Fisher:VarTransform=D') 


#factory.BookMethod(dataloader, ROOT.TMVA.Types.kFisher, 'FisherPCA',
#        '!H:!V:Fisher:VarTransform=P') 

#non-linear !!!  BDT

# Try the following hyperparameters:
# MaxDepth=   1 (stump), 5, 10 (deep tree)
# NTrees=     10 (micro forest), 100, 10000 (big forest)
factory.BookMethod(dataloader, ROOT.TMVA.Types.kBDT, "BDT",
                   ":".join([
                       "!H",
                       "!V",
                       "NTrees=10",
                       #"nEventsMin=150",
                       "MaxDepth=1",
                       "BoostType=AdaBoost",
                       "AdaBoostBeta=0.5",
                       "SeparationType=GiniIndex",
                       #"nCuts=20",
                       "PruneMethod=NoPruning",
                       ]))

factory.BookMethod(dataloader, ROOT.TMVA.Types.kBDT, "BDTB",
                   ":".join([
                       "!H",
                       "!V",
                       "NTrees=10",
                       #"nEventsMin=150",
                       "MaxDepth=1",
                       "BoostType=Bagging",
                       #"AdaBoostBeta=0.5",
                       "SeparationType=GiniIndex",
                       #"nCuts=20",
                       "PruneMethod=NoPruning",
                       ]))
                   
#factory.BookMethod( dataloader, ROOT.TMVA.Types.kLikelihood, "Likelihood",
#                           "H:!V:TransformOutput:PDFInterpol=Spline2:NSmoothSig[0]=20:NSmoothBkg[0]=20:NSmoothBkg[1]=10:NSmooth=1:NAvEvtPerBin=50" )                  
                   
#factory.BookMethod( dataloader, ROOT.TMVA.Types.kLikelihood, "LikelihoodD",
#                           "H:!V:TransformOutput:PDFInterpol=Spline2:NSmoothSig[0]=20:NSmoothBkg[0]=20:NSmoothBkg[1]=10:NSmooth=1:NAvEvtPerBin=50:VarTransform=Decorrelate" )                  
                   
#factory.BookMethod( dataloader, ROOT.TMVA.Types.kLikelihood, "LikelihoodPCA",
#                           "H:!V:TransformOutput:PDFInterpol=Spline2:NSmoothSig[0]=20:NSmoothBkg[0]=20:NSmoothBkg[1]=10:NSmooth=1:NAvEvtPerBin=50:VarTransform=PCA" )                  

# K-Nearest Neighbour classifier (KNN)
#factory.BookMethod( dataloader, ROOT.TMVA.Types.kKNN, "KNN",                           #"H:nkNN=20:ScaleFrac=0.8:SigmaFact=1.0:Kernel=Gaus:UseKernel=F:UseWeight=T:!Trim" )

# PDE_RS
#factory.BookMethod( dataloader, ROOT.TMVA.Types.kPDERS, "PDERS",
                           #"!H:!V:NormTree=T:VolumeRangeMode=Adaptive:KernelEstimator=Gauss:GaussSigma=0.3:NEventsMin=400:NEventsMax=600" )

factory.TrainAllMethods()
factory.TestAllMethods()
factory.EvaluateAllMethods()

#TMVA::TMVAGui()
#ROOT.TMVA.TMVAGui()




